"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getName(type) {
    const name = typeof type == 'function' ? type.name :
        typeof type == 'object' ? type.name :
            type;
    return name;
}
function adapter(type, config) {
    return function (target) {
        const name = getName(type);
        if (config && config.singleton) {
            target.singleton = name;
        }
        else {
            target.provides = name;
        }
    };
}
exports.adapter = adapter;
function port(type) {
    const typeName = getName(type);
    return function (target, propName) {
        const backendName = Symbol.for(propName);
        Reflect.defineProperty(target, backendName, {
            writable: true,
            enumerable: false
        });
        target.constructor.injection = target.constructor.injection || [];
        target.constructor.injection.push({ propName, type: typeName });
        Reflect.defineProperty(target, propName, {
            get: function () {
                if (!this[backendName]) {
                    this[backendName] = injectProp(typeName, []);
                }
                return this[backendName];
            },
            set: function (value) {
                if (!this[backendName]) {
                    this[backendName] = value;
                }
            }
        });
    };
}
exports.port = port;
let registry = {};
function injectProp(name, args) {
    // hack on typescript	
    if (registry[name]) {
        const entry = registry[name];
        if (entry.resolve) {
            return entry.resolve();
        }
        else if (entry.instance) {
            if (typeof entry.instance == 'function') {
                const ctr = entry.instance;
                entry.instance = new ctr(...args);
            }
            return entry.instance;
        }
    }
}
function New(type, ...args) {
    var _a;
    return _a = Container.resolve(type, ...args), (_a !== null && _a !== void 0 ? _a : new type(...args));
}
exports.New = New;
class Container {
    static clear() {
        registry = {};
    }
    static resolve(type, ...args) {
        return injectProp(getName(type), args);
    }
    static registerAs(type, nameOrType) {
        const name = getName(nameOrType); // hack on symbols in dictionaries
        registry[name] = { resolve: () => new type(), instance: null };
    }
    static registerSingleton(instance, nameOrType) {
        const name = getName(nameOrType); // hack on symbols in dictionaries
        registry[name] = { resolve: null, instance };
    }
    static singleton(type, impl) {
        const name = getName(type); // hack on symbols in dictionaries
        registry[name] = { instance: (impl !== null && impl !== void 0 ? impl : type) };
    }
    static register(...types) {
        for (const type of types) {
            let unknow = type;
            if (unknow.provides) {
                registry[unknow.provides] = { resolve: () => new type(), instance: null };
            }
            else if (unknow.singleton) {
                registry[unknow.singleton] = { instance: type, resolve: null };
            }
            else {
                throw new Error("Type " + type.name + " doesn't exports any interface, please use registerAs");
            }
        }
    }
}
exports.Container = Container;
//# sourceMappingURL=index.js.map