"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
// default use
function defaultUsage() {
    class Port {
        lower(text) { throw 'Not implemented'; }
    }
    let Adapter = class Adapter {
        lower(text) {
            return text.toLowerCase();
        }
    };
    Adapter = __decorate([
        index_1.adapter(Port)
    ], Adapter);
    class Library {
        test() {
            if (this.port.lower("UPPER") != "upper") {
                throw new Error("Test failed");
            }
            console.log("Test passed");
        }
    }
    __decorate([
        index_1.port(Port)
    ], Library.prototype, "port", void 0);
    index_1.Container.clear();
    index_1.Container.register(Adapter);
    let lib = new Library();
    lib.test();
}
function usageWithSymbol() {
    const PortSymbol = Symbol("port");
    let Adapter = class Adapter {
        lower(text) {
            return text.toLowerCase();
        }
    };
    Adapter = __decorate([
        index_1.adapter(PortSymbol)
    ], Adapter);
    class Library {
        test() {
            if (this.port.lower("UPPER") != "upper") {
                throw new Error("Symbol test failed");
            }
            console.log("Symbol test passed");
        }
    }
    __decorate([
        index_1.port(PortSymbol)
    ], Library.prototype, "port", void 0);
    index_1.Container.clear();
    index_1.Container.register(Adapter);
    let lib = new Library();
    lib.test();
}
function singletons() {
    const PortSymbol = Symbol("port");
    let Adapter = class Adapter {
        constructor() {
            this.number = 0;
        }
        count() {
            return ++this.number;
        }
    };
    Adapter = __decorate([
        index_1.adapter(PortSymbol, { singleton: true })
    ], Adapter);
    class Library {
        test() {
            return this.port.count();
        }
    }
    __decorate([
        index_1.port(PortSymbol)
    ], Library.prototype, "port", void 0);
    index_1.Container.clear();
    index_1.Container.register(Adapter);
    const lib1 = new Library();
    lib1.test();
    const lib2 = new Library();
    if (lib1.test() != 2) {
        throw new Error("Singleton test1 failed");
    }
    else {
        console.log("Singleton test1 passed");
    }
    index_1.Container.clear();
    index_1.Container.registerSingleton(new Adapter(), PortSymbol);
    const lib3 = new Library();
    lib3.test();
    const lib4 = new Library();
    if (lib4.test() != 2) {
        throw new Error("Singleton test2 failed");
    }
    else {
        console.log("Singleton test2 passed");
    }
}
function usegeWithNameObjects() {
    const Port = {
        name: 'port'
    };
    let Adapter = class Adapter {
        lower(text) {
            return text.toLowerCase();
        }
    };
    Adapter = __decorate([
        index_1.adapter(Port)
    ], Adapter);
    class Library {
        constructor() {
            if (this.port.lower("UPPER") != "upper") {
                throw new Error("Test failed");
            }
            console.log("Usage with names passed");
        }
    }
    __decorate([
        index_1.port(Port)
    ], Library.prototype, "port", void 0);
    index_1.Container.clear();
    index_1.Container.register(Adapter);
    let lib = new Library();
}
usegeWithNameObjects();
defaultUsage();
usageWithSymbol();
singletons();
function newOperator() {
    index_1.Container.clear();
    class Default {
        hello() { return "default"; }
        ;
    }
    index_1.Container.singleton(Default);
    const def1 = index_1.New(Default);
    if (def1.hello() != 'default')
        throw new Error("Error in singleton with new");
    class Derive {
        hello() { return "derive"; }
        ;
    }
    index_1.Container.clear();
    index_1.Container.singleton(Default, Derive);
    const def2 = index_1.New(Default);
    if (def2.hello() != 'derive')
        throw new Error("Error in singleton with new");
}
newOperator();
//# sourceMappingURL=test.js.map